//
// Created by Ari on 10/1/24.
//

#include "HttpConnect.h"

#include <iostream>
#include <string>
#include <cstring>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <arpa/inet.h>
using namespace std;

  string HttpConnect::makeRequest(string host, string path, string port) {
    // Step 1: Create a socket
    int sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock == -1) {
      throw new HttpConnect::ConnectionError("Failed to create socket");
    }

    // Step 2: Resolve hostname to an IP address
    struct addrinfo hints, *res;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_INET;  // IPv4
    hints.ai_socktype = SOCK_STREAM;  // TCP

    if (getaddrinfo(host.c_str(), port.c_str(), &hints, &res) != 0) {
      close(sock);
      throw new HttpConnect::ConnectionError("Failed to resolve hostname");
    }

    // Step 3: Connect to the server
    if (connect(sock, res->ai_addr, res->ai_addrlen) != 0) {
      close(sock);
      throw new HttpConnect::ConnectionError("Failed to connect to server");
    }

    freeaddrinfo(res);  // Free memory that is no longer needed after connecting

    // Step 4: Send HTTP GET request
    std::string request = "GET " + path + " HTTP/1.1\r\n"
                                          "Host: " + host + "\r\n"
                                                            "Connection: close\r\n\r\n";

    if (send(sock, request.c_str(), request.size(), 0) == -1) {
      close(sock);
      throw new HttpConnect::ConnectionError("Failed to send request");
    }

    // Step 5: Receive and print the response
    const int buffer_size = 4096;
    char buffer[buffer_size];
    std::string response;

    ssize_t bytesReceived;
    while ((bytesReceived = recv(sock, buffer, buffer_size - 1, 0)) > 0) {
      buffer[bytesReceived] = '\0';  // Null-terminate the buffer
      response.append(buffer);
    }

    if (bytesReceived < 0) {
      close(sock);
      throw new HttpConnect::ConnectionError("Error receiving response");
    } else {
      close(sock);
      return response;
    }
  }